@@ -123,6 +123,8 @@ def main():
         network_id = network_group.create_network(name=network_name,size=args.size,version=args.version)['id']
         network = netfoundry.Network(network_group, network_id=network_id)
         network.wait_for_status("PROVISIONED",wait=999,progress=True)
+    else:
+        raise Exception("ERROR: failed to find a network named \"{:s}\"".format(network_name))
 
     # existing hosted ERs
     hosted_edge_routers = network.edge_routers(only_hosted=True)
@@ -161,7 +163,7 @@ def main():
             data_center_ids = [dc['id'] for dc in network.aws_geo_regions[region]]
             existing_count = len([er for er in hosted_edge_routers if er['dataCenterId'] in data_center_ids])
             if existing_count < 1:
-                choice = random.choice(network.aws_geo_regions[region])
+                choice = random.choice(network.aws_geo_regions[region]) # nosec
                 # append the current major region to the randomly-chosen data center object
                 #   so we can use it as a role attribute when we create the hosted Edge Router
                 choice['geoRegion'] = region
@@ -245,7 +247,7 @@ def main():
 
     # the demo containers have the demo working dir mounted on /netfoundry
     if os.access('/netfoundry', os.W_OK):
-        token_path = '/netfoundry'
+        token_path = '/netfoundry' # nosec
     else:
         token_path = str(Path.cwd())
     for end in clients+exits:
@@ -300,7 +302,7 @@ def main():
             print("INFO: found Service \"{:s}\"".format(service2['name']))
 
     # Create router-hosted Services unless exists
-    hosting_router = random.choice(hosted_edge_routers)
+    hosting_router = random.choice(hosted_edge_routers) # nosec
 
     service3_name = "Fireworks Service"
     if not service3_name in [svc['name'] for svc in services]:
