@@ -1,4 +1,4 @@
-import json                 # 
+import json
 import requests             # HTTP user agent will not emit server cert warnings if verify=False
 from requests.packages.urllib3.exceptions import InsecureRequestWarning
 requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
@@ -446,7 +446,7 @@ class Network:
             )
         return(resource)
 
-    def create_endpoint(self, name: str, attributes: list=[], session_identity: str=None):
+    def create_endpoint(self, name: str, attributes: list=[], session_identity: str=None, wait: int=10, sleep: int=1, progress: bool=False):
         """create an Endpoint
         :param: name is required string on which to key future operations for this Endpoint
         :param: attributes is an optional list of Endpoint roles of which this Endpoint is a member
@@ -480,21 +480,29 @@ class Network:
             response_code = response.status_code
         except:
             raise
-        if response_code == requests.status_codes.codes.OK: # HTTP 200 (synchronous fulfillment)
+
+        endpoint = None
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['endpoints']['create_responses']):
             try:
                 endpoint = json.loads(response.text)
             except ValueError as e:
-                eprint('ERROR: failed to load {:s} object from POST response'.format("Endpoint"))
-                raise(e)
-        else:
-            raise Exception(
-                'ERROR: got unexpected HTTP code {:s} ({:d}) and response {:s}'.format(
+                raise e('ERROR: failed to load endpoint JSON, got HTTP code {:s} ({:d}) with body {:s}'.format(
                     requests.status_codes._codes[response_code][0].upper(),
                     response_code,
-                    response.text
+                    response.text)
                 )
+                    
+        else:
+            raise Exception('ERROR: got unexpected HTTP code {:s} ({:d}) with body {:s}'.format(
+                    requests.status_codes._codes[response_code][0].upper(),
+                    response_code,
+                    response.text)
             )
 
+        if wait:
+            endpoint = self.wait_for_property_defined(property_name="jwt", property_type=str, entity_type="endpoint", id=endpoint['id'], wait=wait, sleep=sleep, progress=progress)
         return(endpoint)
 
     def create_edge_router(self, name, attributes=[], link_listener=False, data_center_id=None):
@@ -526,7 +534,9 @@ class Network:
             response_code = response.status_code
         except:
             raise
-        if response_code == requests.status_codes.codes[RESOURCES['edge-routers']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['edge-routers']['create_responses']):
             try:
                 router = json.loads(response.text)
             except ValueError as e:
@@ -572,7 +582,9 @@ class Network:
             response_code = response.status_code
         except:
             raise
-        if response_code == requests.status_codes.codes[RESOURCES['edge-router-policies']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['edge-router-policies']['create_responses']):
             try:
                 policy = json.loads(response.text)
             except ValueError as e:
@@ -720,7 +732,9 @@ class Network:
             response_code = response.status_code
         except:
             raise
-        if response_code == requests.status_codes.codes[RESOURCES['services']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['services']['create_responses']):
             try:
                 service = json.loads(response.text)
             except ValueError as e:
@@ -737,27 +751,28 @@ class Network:
 
         return(service)
 
-    def create_endpoint_service(self, name: str, endpoints: list, client_host_names: list, client_port_ranges: list, client_protocols: list=["tcp"], 
-        server_host_name: str=None, server_port: str=None, server_protocol: str=None, attributes: list=[], 
+    def create_endpoint_service(self, name: str, endpoints: list, client_hosts: list, client_ports: list, client_protocols: list=["tcp"], 
+        server_hosts: list=[], server_ports: list=[], server_protocols: list=[], attributes: list=[], 
         edge_router_attributes: list=["#all"], encryption_required: bool=True, dry_run: bool=False):
         """create an Endpoint-hosted Service compatible with Ziti config types intercept.v1, host.v1.
 
         Multiple client intercepts may be specified i.e. lists of domain names or IP addresses, ports, and protocols. If alternative 
         server details are not given they are assumed to be the same as the intercept. If server details are provided then all intercepts 
-        flow to that server.
+        flow to that server. You may constrain the list of allowed server hosts, ports, and protocols
+        that may be forwarded.
 
         :param: name is required string
-        :param: client_host_names is required list of strings that are intercept hostnames (DNS) or IPv4
-        :param: client_port_ranges is required list of strings of the port ranges to intercept as ["80","5900:5999"]
-        :param: client_protocols is required list of strings of the transports. Choices: ["tcp","udp", "sctp"]
-        :param: server_hostname is optional string that is a hostname (DNS) or IPv4. If omitted the client hostname is used.
-        :param: server_port is optional string of the server port. If omitted the same client port is used. 
-        :param: server_protocol is optional string of the server protocol. If omitted the same client protocol is used.
-        :param: attributes is optional list of strings of Service roles to assign. Default is [].
+        :param: client_hosts is required list of strings that are intercept domain name or IPv4.
+        :param: client_ports is required list of strings of the port ranges to intercept as ["80","5900:5999"].
+        :param: client_protocols is optional list of strings of the transports to intercept. Choices: ["tcp","udp"]. Default is ["tcp"].
+        :param: server_hosts is optional list of strings that are a domain name, IPv4, or IPv6. If omitted the client host is used.
+        :param: server_ports is optional list of strings of the port ranges to forward as ["80","5900:5999"]. If omitted same client port is used.
+        :param: server_protocols is optional list of strings of the server protocols to forward. Choices: ["tcp","udp"]. If omitted the client protocol is used.
+        :param: attributes is optional list of strings of Service roles to assign, associating the Service with a matching AppWAN. Default is [].
         :param: edge_router_attributes is optional list of strings of Router roles or Router names that can "see" this Service. Default is ["#all"].
-        :param: endpoints is optional list of strings of hosting Endpoints.
+        :param: endpoints is optional list of strings of Endpoints' #hashtag or @name that will host this Service.
         :param: encryption_required is optional Boolean. Default is to enable edge-to-edge encryption.
-        :param: dry_run is optional Boolean where True returns the entity model without sending a request.
+        :param: dry_run is optional Boolean where True returns only the entity model without sending a request.
         """
         try:
             # validate roles
@@ -768,12 +783,11 @@ class Network:
             # these elements could be a single port as integer or a string with one or two ports separated by a : or -
             separators = re.compile('[:-]')
             valid_range = re.compile('^\d+[:-]\d+$')
-            valid_client_port_ranges = list()
-            for range in client_port_ranges:
-                # if an integer or single port then use same number for low:high
-                if isinstance(range, int):
+            valid_client_ports = list()
+            for range in client_ports:
+                if isinstance(range, int): # if an integer or single port then use same number for low:high
                     range = str(range)+':'+str(range)
-                elif not re.search(separators,range):
+                elif not re.search(separators,range): # if a string of a single port (no : or - separator) then cat with separator :
                     range = range+':'+range
                 
                 if not re.fullmatch(valid_range, range):
@@ -783,38 +797,68 @@ class Network:
                 if not len(bounds) == 2 or int(bounds[0]) > int(bounds[1]):
                     raise Exception("ERROR: failed to find one lower, one higher port for range: {}".format(range))
                 else:
-                    valid_client_port_ranges.append({"low": bounds[0], "high": bounds[1]})
+                    valid_client_ports.append({"low": bounds[0], "high": bounds[1]})
+            valid_server_ports = list()
+            for range in server_ports:
+                if isinstance(range, int): # if an integer or single port then use same number for low:high
+                    range = str(range)+':'+str(range)
+                elif not re.search(separators,range): # if a string of a single port (no : or - separator) then cat with separator :
+                    range = range+':'+range
+                
+                if not re.fullmatch(valid_range, range):
+                    raise Exception("ERROR: failed to parse server port range: {}".format(range))
+
+                bounds = re.split(separators, range)
+                if not len(bounds) == 2 or int(bounds[0]) > int(bounds[1]):
+                    raise Exception("ERROR: expecting a port or port range, got: {range}".format(range=range))
+                else:
+                    valid_server_ports.append({"low": bounds[0], "high": bounds[1]})
 
             # validate client protocols
             valid_client_protocols = list()
             for proto in client_protocols:
-                if not proto in ["tcp", "udp", "sctp", "TCP", "UDP", "SCTP"]:
-                    raise Exception("ERROR: client intercept protocol \"{}\" is not valid.".format(proto))
+                if not proto.lower() in ["tcp", "udp"]:
+                    raise Exception("ERROR: client intercept protocol \"{}\" is not valid.".format(proto.lower()))
                 else:
                     valid_client_protocols.append(proto.lower())
 
+            # validate server protocols
+            valid_server_protocols = list()
+            for proto in server_protocols:
+                if not proto.lower() in ["tcp", "udp"]:
+                    raise Exception("ERROR: server protocol \"{}\" is not valid.".format(proto.lower()))
+                else:
+                    valid_server_protocols.append(proto.lower())
+
             # resolve exit hosting params
             server_egress = dict()
-            if server_host_name:
-                server_egress["address"] = server_host_name
-                server_egress["dialInterceptAddress"] = None
+            if server_hosts and len(server_hosts) == 1:
+                server_egress["host"] = server_hosts[0]
+            elif server_hosts and len(server_hosts) > 1:
+                server_egress["forwardHost"] = True
+                server_egress["allowedHosts"] = server_hosts
             else:
-                server_egress["dialInterceptAddress"] = True
-                server_egress["address"] = None
+                server_egress["forwardHost"] = True
+                server_egress["allowedHosts"] = client_hosts
 
-            if server_port:
-                server_egress["port"] = server_port
-                server_egress["dialInterceptPort"] = None
+            if valid_server_ports and len(valid_server_ports) == 1 and valid_server_ports[0]['low'] == valid_server_ports[0]['high']:
+                server_egress["port"] = valid_server_ports[0]['low']
+            elif valid_server_ports:
+                server_egress["forwardPort"] = True
+                server_egress["allowedPortRanges"] = valid_server_ports
             else:
-                server_egress["dialInterceptPort"] = True
-                server_egress["port"] = None
+                server_egress["forwardPort"] = True
+                server_egress["allowedPortRanges"] = valid_client_ports
 
-            if server_protocol:
-                server_egress["protocol"] = server_protocol
-                server_egress["dialInterceptProtocol"] = None
+            if server_protocols and len(server_protocols) == 1:
+                server_egress["protocol"] = server_protocols[0]
+            elif server_protocols and len(server_protocols) > 1:
+                server_egress["forwardProtocol"] = True
+                server_egress["allowedProtocols"] = server_protocols
             else:
-                server_egress["dialInterceptProtocol"] = True
-                server_egress["protocol"] = None
+                server_egress["forwardProtocol"] = True
+                server_egress["allowedProtocols"] = client_protocols
+
 
             # parse out the elements in the list of endpoints as one of #attribute, UUID, or resolvable Endoint name
             bind_endpoints = list()
@@ -857,8 +901,8 @@ class Network:
                 "model": {
                     "bindEndpointAttributes": bind_endpoints,
                     "clientIngress" : {
-                        "addresses": client_host_names, 
-                        "ports": valid_client_port_ranges,
+                        "addresses": client_hosts, 
+                        "ports": valid_client_ports,
                         "protocols": valid_client_protocols
                     },
                     "serverEgress": server_egress,
@@ -887,7 +931,9 @@ class Network:
         except:
             raise
 
-        if response_code == requests.status_codes.codes[RESOURCES['services']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['services']['create_responses']):
             try:
                 service = json.loads(response.text)
             except ValueError as e:
@@ -932,7 +978,9 @@ class Network:
             response_code = response.status_code
         except:
             raise
-        if response_code == requests.status_codes.codes[RESOURCES['app-wans']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['app-wans']['create_responses']):
             try:
                 app_wan = json.loads(response.text)
             except ValueError as e:
@@ -1031,6 +1079,84 @@ class Network:
 
         return(network)
 
+    def wait_for_property_defined(self, property_name: str, property_type: object=str, entity_type: str="network", wait: int=300, sleep: int=20, id: str=None, progress: bool=False):
+        """continuously poll until expiry for the expected property to become defined with the any value of the expected type
+        :param property_name: a top-level property to wait for e.g. `zitiId`
+        :param property_type: optional Python instance type to expect for the value of property_name
+        :param id: the UUID of the entity having a status if entity is not a network
+        :param entity_type: optional type of entity e.g. network (default), endpoint, service, edge-router
+        :param wait: optional SECONDS after which to raise an exception defaults to five minutes (300)
+        :param sleep: SECONDS polling interval
+        """
+
+        # use the id of this instance's Network unless another one is specified
+        if entity_type == "network" and not id:
+            id = self.id
+
+        now = time.time()
+
+        if not wait >= sleep:
+            raise Exception(
+                "ERROR: wait duration ({:d}) must be greater than or equal to polling interval ({:d})".format(
+                    wait, sleep
+                )
+            )
+
+        # poll for status until expiry
+        if progress:
+            sys.stdout.write(
+                '\twaiting for property {:s} ({:s}) or until {:s}.'.format(
+                    property_name,
+                    str(property_type),
+                    time.ctime(now+wait)
+                )
+            )
+
+#        response_code = int()
+        property_value = None
+        while time.time() < now+wait:
+            if progress:
+                sys.stdout.write('.') # print a stop each iteration to imply progress
+                sys.stdout.flush()
+
+            try:
+                entity = self.get_resource(type=entity_type, id=id)
+            except:
+                raise
+
+            # if expected property value is not null then evaluate type, else sleep
+            if property_name in entity.keys() and entity[property_name]:
+                property_value = entity[property_name]
+                # if expected type then return, else sleep
+                if isinstance(property_value, property_type):
+                    if progress:
+                        print() # newline terminates progress meter
+                    return(entity)
+                else:
+                    if progress:
+                        sys.stdout.write('\n{:^19s}:{:^19s} ({:s}):'.format(entity['name'],property_name, str(property_type)))
+                    time.sleep(sleep)
+            else:
+                if progress:
+                    sys.stdout.write('\n{:^19s}:{:^19s} ({:s}):'.format("fetching",property_name, str(property_type)))
+                time.sleep(sleep)
+
+        # 
+        if progress:
+            print() # newline terminates progress meter
+
+        if not property_value:
+            raise Exception('ERROR: failed to find any value for property "{:s}"'.format(
+                    property_name
+                )
+            )
+        else:
+            raise Exception('ERROR: timed out waiting for property {:s} to have expected type: {:s}'.format(
+                    property_name,
+                    str(property_type),
+                )
+            )
+
     def wait_for_status(self, expect: str="PROVISIONED", type: str="network", wait: int=300, sleep: int=20, id: str=None, progress: bool=False):
         """continuously poll for the expected status until expiry
         :param expect: the expected status symbol e.g. PROVISIONED
