@@ -137,7 +137,7 @@ class NetworkGroup:
 
         return(aws_data_centers)
 
-    def get_product_metadata(self):
+    def get_product_metadata(self, is_active: bool=True):
         """fetch all product metadata
         """
         try:
@@ -165,7 +165,14 @@ class NetworkGroup:
                 )
             )
 
-        return (product_metadata)
+        if is_active:
+            active_product_metadata = dict()
+            for product in product_metadata.keys():
+                if product_metadata[product]['active']:
+                    active_product_metadata[product] = product_metadata[product]
+            return (active_product_metadata)
+        else:
+            return (product_metadata)
 
     def list_product_versions(self, product_metadata: dict={}):
         """find all product version from product metadata
@@ -243,7 +250,20 @@ class NetworkGroup:
         except:
             raise
 
-        if not response_code == requests.status_codes.codes[RESOURCES['networks']['expect']]:
+        any_in = lambda a, b: any(i in b for i in a)
+        response_code_symbols = [s.upper() for s in requests.status_codes._codes[response_code]]
+        if any_in(response_code_symbols, RESOURCES['networks']['create_responses']):
+            try:
+                network = json.loads(response.text)
+            except ValueError as e:
+                raise e('ERROR: failed to load created network JSON, got HTTP code {:s} ({:d}) with body {:s}'.format(
+                    requests.status_codes._codes[response_code][0].upper(),
+                    response_code,
+                    response.text)
+                )
+            else:
+                return(network)
+        else:
             raise Exception(
                 'ERROR: got unexpected HTTP code {:s} ({:d}) and response {:s}'.format(
                     requests.status_codes._codes[response_code][0].upper(),
@@ -252,9 +272,6 @@ class NetworkGroup:
                 )
             )
 
-        network = json.loads(response.text)
-        return(network)
-
     def delete_network(self, network_id=None, network_name=None):
         """
         delete a Network
